==> tcputils-0.5.tar.gz: tcputils-0.5/tcpconnect.c <==
/*
 *  tcpconnect.c
 *		Connect to a TCP/IP port, send standard input to that
 *		connection, and print messages from that connection on
 *		standard output.
 *
 *
 *  This program is in the public domain.  You may do anything
 *  you like with it.
 *
 *
 *  Author:	Thomas Bellman
 *		Lysator Computer Club
 *		Linkping University
 *		Sweden
 *  email:	Bellman@Lysator.LiU.Se
 *
 *
 *  Any opinions expressed in this code are the author's PERSONAL opinions,
 *  and does NOT, repeat NOT, represent any official standpoint of Lysator,
 *  even if so stated.
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdarg.h>

#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/filio.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "ip/ip_misc.h"


extern	int	getopt(int, char *const*, const char *);
extern	int	optind;


char	* progname		= "tcpconnect";
int	  debug			= 0;
int	  quiet_mode		= 0;


void
usage(void)
{
    fprintf(stderr, "Usage: %s [-dlrq] host port\n", progname);
    exit(1);
}


void
log(const char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);
    if (!quiet_mode)
	vfprintf(stderr, fmt, args);
    va_end(args);
}


void
fatal(char	* msg)
{
    extern char	* sys_errlist[];

    fprintf(stderr, "%s: %s: %s\n", progname, msg, sys_errlist[errno]);
    exit(1);
}



#define MIN(a,b)	((a) < (b) ? (a) : (b))

static	long
drain(int from,
      int to)
{
    char buffer[8192];
    long bytes_read = 0;
    long avail;
    long bytes;
    long nreads = 0;
    
    while (1)
    {
	ioctl(from, FIONREAD, &avail);
	if (avail == 0)
	    break;
	bytes = read(from, buffer, MIN(avail, sizeof buffer));
	if (bytes < 0)
	    return -1;
	if (bytes == 0)
	    break;
	nreads++;
	bytes_read += bytes;
	write(to, buffer, bytes);
    }
    return bytes_read;
}



struct relay {
	int	source;
	int	dest;
	int	terminate_on_eof;
};


static	int
relay_once(struct relay		* relaylist,
	   int			  nrelays,
	   struct timeval	* timeout,
	   int			* neofs)
{
    int maxfd;
    fd_set readset;
    int i;
    int nfds;

    if (neofs)
	*neofs = 0;

    FD_ZERO(&readset);
    maxfd = 0;
    for (i = nrelays - 1 ;  i >= 0 ;  i--) 
    {
	register int from = relaylist[i].source;
	if (from >= 0)
	    FD_SET(from, &readset);
	if (from > maxfd)
	    maxfd = from;
    }

    nfds = select(maxfd+1, &readset, (fd_set*)NULL, (fd_set*)NULL, timeout);
    if (nfds <= 0) {
	if (errno != EINTR)
	    return nfds;
	else
	    return 0;
    }

    for (i = nrelays - 1 ;  i >= 0 ;  i--)
    {
	register int *from = &relaylist[i].source;
	register int *to = &relaylist[i].dest;

	if (*from >= 0  &&  FD_ISSET(*from, &readset))
	{
	    register long bytes_copied = drain(*from, *to);
	    if (bytes_copied <= 0)
	    {
		char minibuf;

		read(*from, &minibuf, sizeof minibuf);
		*from = -1;
		if (relaylist[i].terminate_on_eof  &&  neofs)
		    ++*neofs;
	    }
	}
    }

    return nfds;
}


static	int
relay(struct relay	* relaylist,
      int		  nrelays,
      struct timeval	* timeout)
{
    int terminate;
    struct timeval	  my_timeout;
    struct timeval	* my_timeoutptr = &my_timeout;

    do
    {
	if (timeout)
	    memcpy(&my_timeout, timeout, sizeof my_timeout);
	else
	    my_timeoutptr = NULL;
	relay_once(relaylist, nrelays, my_timeoutptr, &terminate);
    } while (!terminate);

    return 0;
}

	


int
main (int	   argc,
      char	** argv)

{
    int		  tcp;
    int		  option;
    int		  close_on_stdin_eof	= 0;
    int		  close_on_sock_eof	= 0;

    progname = argv[0];
    while ((option = getopt (argc, argv, "dlrq")) != EOF)
    {
	switch ((char)option)
	{
	case 'd':
	    debug++;
	    break;
	case 'l':
	    close_on_stdin_eof = 1;
	    break;
	case 'r':
	    close_on_sock_eof = 1;
	    break;
	case 'q':
	    quiet_mode = 1;
	    break;
	default:
	    usage();
	    /* NOTREACHED */
	}
    }

    if (argc - optind != 2)
	usage();
    
    {
	struct sockaddr_in	  server;
	int			  serversize	= sizeof server;
	struct relay		  fds[2];
	
	tcp = tcp_open(argv[optind], argv[optind+1], NULL, NULL);
	if (tcp < 0)
	    fatal("Can't connect to server");
	if (getpeername(tcp, (struct sockaddr*)&server, &serversize) >= 0)
	    log("Connected to %s port %d\n",
		inet_ntoa(server.sin_addr), ntohs(server.sin_port));
	else
	    log("Connected to unknown server\n");

	fds[0].source = 0;
	fds[0].dest = tcp;
	fds[0].terminate_on_eof = close_on_stdin_eof || !close_on_sock_eof;

	fds[1].source = tcp;
	fds[1].dest = 1;
	fds[1].terminate_on_eof = close_on_sock_eof || !close_on_stdin_eof;

	relay(fds, 2, NULL);
    }
    
    return 0;
}

==> tcputils-0.6.tar.gz: tcputils-0.6/tcpconnect.c <==
/*
 *  tcpconnect.c
 *		Connect to a TCP/IP port, send standard input to that
 *		connection, and print messages from that connection on
 *		standard output.
 *
 *
 *  This program is in the public domain.  You may do anything
 *  you like with it.
 *
 *
 *  Author:	Thomas Bellman
 *		Lysator Computer Club
 *		Linkping University
 *		Sweden
 *  email:	Bellman@Lysator.LiU.Se
 *
 *
 *  Any opinions expressed in this code are the author's PERSONAL opinions,
 *  and does NOT, repeat NOT, represent any official standpoint of Lysator,
 *  even if so stated.
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdarg.h>
#include <signal.h>

#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "ip/ip_misc.h"
#include "relay.h"


extern	int	 getopt(int, char *const*, const char *);
extern	int	 optind;
extern	char	*optarg;


char	* progname		= "tcpconnect";
int	  verbose		= 0;


void
usage(void)
{
    fprintf(stderr, "Usage: %s [-irv] [-l addr:port] host port\n", progname);
    exit(1);
}


void
log(const char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);
    if (verbose)
	vfprintf(stderr, fmt, args);
    va_end(args);
}


void
fatal(char	* msg)
{
    extern char	* sys_errlist[];

    fprintf(stderr, "%s: %s: %s\n", progname, msg, sys_errlist[errno]);
    exit(1);
}



int
main (int	   argc,
      char	** argv)

{
    int		  tcp;
    int		  option;
    int		  close_on_stdin_eof	= 0;
    int		  close_on_remote_eof	= 0;
    char	* localaddr		= NULL;
    char	* localport		= NULL;

    progname = argv[0];
    while ((option = getopt(argc, argv, "irvl:q")) != EOF)
    {
	switch ((char)option)
	{
	case 'i':
	    close_on_stdin_eof = 1;
	    break;
	case 'r':
	    close_on_remote_eof = 1;
	    break;
	case 'l':
	    localaddr = optarg;
	    if ((localport = strrchr(localaddr, ':')) != NULL) {
		if (localaddr == localport)
		    localaddr = NULL;
		*localport = '\0';
		localport++;
	    }
	    break;
	case 'v':
	    verbose = 1;
	    break;
	default:
	    usage();
	    /* NOTREACHED */
	}
    }

    if (argc - optind != 2)
	usage();
    
    {
	struct sockaddr_in	  server;
	int			  serversize	= sizeof server;
	struct relay		  fds[2];
	int			  nclosed	= 0;

	tcp = tcp_open(argv[optind], argv[optind+1], localaddr, localport);
	if (tcp < 0)
	    fatal("Can't connect to server");
	if (getpeername(tcp, (struct sockaddr*)&server, &serversize) >= 0)
	    log("[Connected to %s port %d]\n",
		inet_ntoa(server.sin_addr), ntohs(server.sin_port));
	else
	    log("[Connected to unknown server]\n");

	fds[0].source = 0;
	fds[0].dest = tcp;
	fds[0].writeerror = 0;
	fds[0].readerror = 0;

	fds[1].source = tcp;
	fds[1].dest = 1;
	fds[1].writeerror = 0;
	fds[1].readerror = 0;

	signal(SIGPIPE, SIG_IGN);
	do {
	    int status = relay_once(fds, 2, NULL, NULL);
	    if (status < 0)
		nclosed += -status;
	    if (close_on_stdin_eof  &&
		(fds[0].readerror  ||  fds[1].writeerror))
	    {
		break;
	    }
	    if (close_on_remote_eof  &&
		(fds[1].readerror  ||  fds[0].writeerror))
	    {
		break;
	    }
	} while (nclosed < 2);
	log("[Connection closed]\n");
    }
    
    return 0;
}
